@@ -123,7 +123,10 @@ static void sig_catcher(int sig)
 	}
 #endif
 	if (sig == SIGABRT)
+	{
 		syslog(LOG_CRIT, "aborting %s\n", err_prefix);
+		abort();
+	}
 	caught_signal = sig;
 }
 
@@ -318,6 +321,17 @@ daemon_loop(struct nfq_handle *h, DB *db)
 	ipaddr_cnt cb_cnt;
 	memset(&cb_cnt, 0, sizeof cb_cnt);
 
+	int const fd = nfq_fd(h);
+	int rv;
+	char buf[8192];
+	
+	if (fd < 0)
+	{
+		report_error(&ap, LOG_CRIT, "invalid fd %d (%s)\n",
+					fd, strerror(errno));
+		return 1;
+	}
+
 	for (size_t i = 0; i < queue_data_cnt; ++i)
 	{
 		queue_descr *const qu = &queue_data[i];
@@ -338,16 +352,14 @@ daemon_loop(struct nfq_handle *h, DB *db)
 			nfq_destroy_queue(qh);
 			return 1;
 		}
+		
+		// TODO nfq_set_queue_maxlen(qh, configured_value), if needed
 
 		qu->cnt = &cb_cnt;
 		qu->qh = qh;
 		qu->deadbeef = 0xdeadbeef;
 	}
 
-	int fd = nfq_fd(h);
-	int rv;
-	char buf[8192];
-
 	while (caught_signal == 0 &&
 		(rv = recv(fd, buf, sizeof(buf), 0)) >= 0)
 	{
@@ -468,6 +480,38 @@ daemon_loop(struct nfq_handle *h, DB *db)
 			}
 		}
 	}
+	
+	/*
+	* rv < 0 on EINTR or other error; EINTR implies a signal
+	*/
+	if (caught_signal)
+	{
+		if (caught_signal == SIGHUP)
+		{
+			report_error(&ap, LOG_INFO, "reopening on SIGHUP\n");
+			caught_signal = 0;
+			rv = -1; // enter daemon_loop again
+		}
+		else
+		{
+			report_error(&ap, LOG_INFO, "exiting on signal %s\n",
+				strsignal(caught_signal));
+			rv = 0;  // program exit code
+		}
+	}
+	else
+	{
+		if (errno == EBADF && nfq_fd(h) == -123) // test in-file closed
+		{
+			rv = 0;
+		}
+		else
+		{
+			report_error(&ap, LOG_CRIT, "break loop after recv returns %d (%s)\n",
+				rv, strerror(errno));
+			rv = -1; // enter daemon_loop again, hopefully the error is transient
+		}
+	}
 
 	for (size_t i = 0; i < queue_data_cnt; ++i)
 	{
@@ -481,22 +525,6 @@ daemon_loop(struct nfq_handle *h, DB *db)
 		}
 	}
 
-	// rv < 0 for EINTR
-	rv = 0;
-
-	if (caught_signal)
-	{
-		if (caught_signal == SIGHUP)
-		{
-			rv = -1;
-			caught_signal = 0;
-			report_error(&ap, LOG_INFO, "reopening on SIGHUP\n");
-		}
-		else
-			report_error(&ap, LOG_INFO, "exiting on signal %s\n",
-				strsignal(caught_signal));
-	}
-
 	return rv;
 }
 
