@@ -19,7 +19,7 @@ You should have received a copy of the GNU General Public License
 along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 */
-
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -62,6 +62,23 @@ void report_error(app_private *ap, int pri, char const *fmt, ...)
 	va_end(al);
 }
 
+static int isdir(char const *p)
+{
+	struct stat st;
+	int rtc = stat(p, &st);
+
+	if (rtc == 0)
+	{
+		if (!(S_ISDIR(st.st_mode)))
+		{
+			rtc = -1;
+			errno = ENOTDIR;
+		}
+	}
+	
+	return rtc;
+}
+
 static int createdir (char *fname)
 {
 	int rtc = 0;
@@ -71,7 +88,7 @@ static int createdir (char *fname)
 		if (p == fname)
 			++p;
 		*p = 0;
-		if ((rtc = access(fname, R_OK|W_OK|X_OK)) != 0 &&
+		if ((rtc = isdir(fname)) != 0 &&
 			(errno == ENOENT || errno == ENOTDIR) &&
 			(rtc = createdir(fname)) == 0)
 				rtc = mkdir(fname, 0700);
