@@ -28,6 +28,29 @@ of an old libnetfilter_queue distribution. The content of the original files
 has been completely faked, so that the result cannot serve the purpose of the
 original code. This only serves to link a test program that so that it can run
 without root privileges and without disturbing the kernel.
+
+HOW THIS WORKS:
+===============
+
+The main function runs in the client. It is assumed it calls nfq_open as a
+first thing. Sanity checks are run there, it also retrieves an environment
+variable that has the name of an in-file that with test details, and creates
+the nfq_handle structure filled with the data for the test.
+
+The in-file may contain packets, commands, and comments. Commands are defined
+in cmd_strings.h; the "group" command groups packets. In-file is first read
+on opening nfq_handle, and continues until there are enough packets to be sent.
+
+The socketpair is used to pass parsed in-file data to the client. A call to
+nfq_handle_packet is expected after sending. That function executes any command
+and then calls all registered callbacks. The noop command is used to yield
+control to the client. Other commands are useful to set test values, e.g. the
+database, in order for the client to take certain actions. The client's
+decision is displayed when it calls nfq_set_verdict or similar functions.
+
+When the in-file terminates, the socketpair is closed, letting the client's
+call to recv terminate with EBADF. nfq_fd will return -123 after that.
+
 */
 
 #define _GNU_SOURCE
@@ -633,7 +656,7 @@ struct nfnl_handle *nfq_nfnlh(struct nfq_handle *h)
 
 int nfq_fd(struct nfq_handle *h)
 {
-	return h->sv[1];
+	return h->sv_open? h->sv[1]: -123;
 }
 
 struct nfq_handle *nfq_open(void)
